import os
import sys
import numpy as np
from amuse.test.amusetest import TestWithMPI

from amuse.community.mercury.interface import MercuryInterface, MercuryWayWard,Mercury
from amuse.ext.solarsystem import new_solar_system_for_mercury,new_solar_system
from amuse.units import nbody_system
from amuse.units import units, constants
from amuse import datamodel
from amuse.ic import plummer
DUMMYID=0

try:
    from matplotlib import pyplot
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False

class TestMercuryInterface(TestWithMPI):
    
    def setUp(self):
        super(TestWithMPI, self).setUp()
            
    def test1(self):
        instance=MercuryInterface()  
        instance.stop()

    def test2(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        instance.stop()

    def test3(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        instance.cleanup_code()  
        instance.stop()

    def test4(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        mass,err=instance.get_central_mass(DUMMYID)
        self.assertEqual(mass,1.0)
        radius,err=instance.get_central_radius(DUMMYID)
        self.assertEqual(radius,.005)
        j2,j4,j6,err=instance.get_central_oblateness(DUMMYID)
        self.assertEqual(j2,0.0)
        self.assertEqual(j4,0.0)
        self.assertEqual(j6,0.0)
        lx,ly,lz,err=instance.get_central_spin(DUMMYID)
        self.assertEqual(lx,0.0)
        self.assertEqual(ly,0.0)
        self.assertEqual(lz,0.0)
        instance.stop()

    def test5(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        instance.set_central_mass(DUMMYID, 2.0)
        instance.set_central_radius(DUMMYID,0.0625)
        instance.set_central_oblateness(DUMMYID,0.001,0.002,0.003)
        instance.set_central_spin(DUMMYID,-0.1,-0.2,-0.3)


        mass,err=instance.get_central_mass(DUMMYID)
        self.assertEqual(mass,2.0)
        radius,err=instance.get_central_radius(DUMMYID)
        self.assertEqual(radius,.0625)
        j2,j4,j6,err=instance.get_central_oblateness(DUMMYID)
        self.assertEqual(j2,0.001)
        self.assertEqual(j4,0.002)
        self.assertEqual(j6,0.003)
        lx,ly,lz,err=instance.get_central_spin(DUMMYID)
        self.assertEqual(lx,-0.1)
        self.assertEqual(ly,-0.2)
        self.assertEqual(lz,-0.3)
        instance.stop()

    def test6(self):
        instance=MercuryInterface()
        instance.initialize_code()
        instance.commit_particles()
        instance.cleanup_code()  
        instance.stop()

    def test7(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        time,err=instance.get_time()  
        self.assertEqual(time,0.0)
        instance.stop()

    def test8(self):
        instance=MercuryInterface()
        instance.initialize_code()  
        n,err=instance.get_number_of_orbiters()  
        self.assertEqual(n,0)
        instance.stop()

    def test9(self):
        instance=MercuryInterface()
        instance.initialize_code()
        pid,err=instance.new_orbiter(0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.)  
        n,err=instance.get_number_of_orbiters()  
        self.assertEqual(n,1)
        instance.stop()

    def test10(self):
        instance=MercuryInterface()
        instance.initialize_code()
        pid,err=instance.new_orbiter(0.5,1.,2.,3.,4.,5.,6.,7.,1.,2.,3.,8.)  
        mass,dens,x,y,z,vx,vy,vz,sx,sy,sz,celimit,err=instance.get_orbiter_state(pid)
        self.assertEqual(mass,0.5)       
        self.assertEqual(dens,1.)       
        self.assertEqual(x,2.)       
        self.assertEqual(y,3.)       
        self.assertEqual(z,4.)
        self.assertEqual(vx,5.)
        self.assertEqual(vy,6.)
        self.assertEqual(vz,7.)
        self.assertEqual(sx,1.)
        self.assertEqual(sy,2.)
        self.assertEqual(sz,3.)
        self.assertEqual(celimit,8.)       
        instance.stop()

    def test11(self):
        instance=MercuryInterface()
        instance.initialize_code()
        mass=3.04043264264672381E-06
        dens=5.52
        x=2.42093942183383037E-01
        y=-9.87467766698604366E-01
        z=-4.54276292555233496E-06
        vx=1.64294055023289365E-02
        vy=4.03200725816140870E-03
        vz=1.13609607260006795E-08
        sx=sy=sz=0.
        celimit=20.
        pid,err=instance.new_orbiter(mass,dens,x,y,z,vx,vy,vz,sx,sy,sz,celimit)  
        instance.commit_particles()
        de,err=instance.get_energy_deviation()
        self.assertEqual(de,0.0)
        etot,err=instance.get_total_energy()
        ep,err=instance.get_potential_energy()
        ek,err=instance.get_kinetic_energy()
        self.assertAlmostEqual(etot,ep+ek,15)
        instance.stop()

    def test12(self):
        instance=MercuryInterface()
        instance.initialize_code()
        mass=3.04043264264672381E-06
        dens=5.52
        x=2.42093942183383037E-01
        y=-9.87467766698604366E-01
        z=-4.54276292555233496E-06
        vx=1.64294055023289365E-02
        vy=4.03200725816140870E-03
        vz=1.13609607260006795E-08
        sx=sy=sz=0.
        celimit=20.
        pid,err=instance.new_orbiter(mass,dens,x,y,z,vx,vy,vz,sx,sy,sz,celimit)  
        instance.commit_particles()
        etot1,err=instance.get_total_energy()
        err=instance.evolve_model(8*100000.)
        etot2,err=instance.get_total_energy()
        self.assertAlmostEqual(etot1,etot2,7)
        instance.stop()

    def test13(self):
        instance=MercuryInterface()
        instance.initialize_code()
        mass=3.04043264264672381E-06
        dens=5.52
        x=2.42093942183383037E-01
        y=-9.87467766698604366E-01
        z=-4.54276292555233496E-06
        vx=1.64294055023289365E-02
        vy=4.03200725816140870E-03
        vz=1.13609607260006795E-08
        sx=sy=sz=0.
        celimit=20.
        pid,err=instance.new_orbiter(mass,dens,x,y,z,vx,vy,vz,sx,sy,sz,celimit)  
        instance.commit_particles()
        instance.enable_stopping_condition(3)
        instance.set_stopping_condition_timeout_parameter(0.01)
        etot1,err=instance.get_total_energy()
        err=instance.evolve_model(8*1e100)
        self.assertTrue(instance.is_stopping_condition_set(3)['result']==1)
        etot2,err=instance.get_total_energy()
        self.assertAlmostEqual(etot1,etot2,7)
        instance.stop()

    def test14(self):
        instance=MercuryInterface(redirection='none')
        instance.initialize_code()  
        instance.set_central_particle_state(DUMMYID, 1.,2.,3.,4.,5.,6.,7.,8.)
        mass,radius,j2,j4,j6,lx,ly,lz,err=instance.get_central_particle_state(DUMMYID)
        self.assertEqual(mass, 1.0)
        self.assertEqual(radius, 2.0)
        self.assertEqual(j2, 3.0)
        self.assertEqual(j4, 4.0)
        self.assertEqual(j6, 5.0)
        self.assertEqual(lx, 6.0)
        self.assertEqual(ly, 7.0)
        self.assertEqual(lz, 8.0)
        instance.stop()

    def test15(self):
        instance=MercuryInterface(redirection='none')
        instance.initialize_code()  
        i,err=instance.get_integrator()
        self.assertEqual(i,10)
        err=instance.set_integrator(2)
        self.assertEqual(err,0)
        i,err=instance.get_integrator()
        self.assertEqual(i,2)
        err=instance.set_integrator(22)
        self.assertEqual(err,-1)
        instance.stop()

    def test16(self):
        instance=MercuryInterface()
        instance.initialize_code()
        err=instance.set_integrator(2)
        mass=3.04043264264672381E-06
        dens=5.52
        x=2.42093942183383037E-01
        y=-9.87467766698604366E-01
        z=-4.54276292555233496E-06
        vx=1.64294055023289365E-02
        vy=4.03200725816140870E-03
        vz=1.13609607260006795E-08
        sx=sy=sz=0.
        celimit=20.
        pid,err=instance.new_orbiter(mass,dens,x,y,z,vx,vy,vz,sx,sy,sz,celimit)  
        instance.commit_particles()
        instance.enable_stopping_condition(3)
        instance.set_stopping_condition_timeout_parameter(0.01)
        etot1,err=instance.get_total_energy()
        err=instance.evolve_model(8*1e100)
        self.assertTrue(instance.is_stopping_condition_set(3)['result']==1)
        etot2,err=instance.get_total_energy()
        self.assertAlmostEqual(etot1,etot2,7)
        instance.stop()


class TestMercury(TestWithMPI):
                
    def setUp(self):
        super(TestWithMPI, self).setUp()
        
    def sun_and_earth(self):
        orbiter = datamodel.Particles(1)
        orbiter.mass = 5.97e24 | units.kg
        orbiter.density = 1.0|units.g/units.cm**3
        orbiter.position = [1.0,0.0,0.0] | units.AU
        orbiter.velocity = [0.0, 2.0*3.1415926535*1.0/365, 0.0] | units.AUd
        orbiter.angular_momentum = [1.0,0,0] | units.MSun * units.AU**2/units.day
        orbiter.celimit = 0.0 | units.none
        
        centre = datamodel.Particles(1)
        centre.mass = 1.0 | units.MSun
        centre.radius = .0000001 | units.AU
        centre.j2 = .0001|units.AU**2
        centre.j4 = .0|units.AU**4
        centre.j6 = .0|units.AU**6
        
        centre.angular_momentum = [0.0, 0.0, 0.0] | units.MSun * units.AU**2/units.day

        return centre, orbiter

    def test0(self):
        centre, orbiter = self.sun_and_earth()
        mercury = MercuryWayWard()#,debugger='xterm')
        mercury.initialize_code()
        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiter)
        mercury.commit_particles()

        self.assertAlmostEqual(mercury.central_particle.j4, .0|units.AU**4)
        self.assertAlmostEqual(mercury.central_particle.mass, 1.98892e+30 |units.kg, 3)
        self.assertAlmostEqual(mercury.central_particle.mass, 1.0 |units.MSun, 3)
        self.assertEqual(mercury.get_number_of_orbiters(),1)
        self.assertEqual(mercury.orbiters.position, [[1,0,0]] | units.AU)
        self.assertEqual(mercury.orbiters.density, 1.0|units.g/units.cm**3 )
        self.assertEqual(mercury.orbiters.angular_momentum, [[1.0, 0.0, 0.0]] | units.MSun*units.AU**2/units.day)

        mercury.evolve_model(365.24 | units.day)

        self.assertAlmostEqual(mercury.orbiters.position, [[1.0, 0.0, 0.0]] | units.AU, 1)
        self.assertAlmostEqual(mercury.kinetic_energy+mercury.potential_energy,mercury.total_energy,3)

        mercury.stop()

    def test1(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        mercury.initialize_code()

        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiters)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()

    def test2(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        mercury.initialize_code()

        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiters)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(16.|units.day)
        self.assertEqual(mercury.get_time(),16.|units.day)
        mercury.stop()

    def test3(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.initialize_code()

        mercury.particles.add_particles(solsys)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()
            
    def test4(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.initialize_code()
        self.assertEqual(mercury.parameters.timestep, 8 | units.day)
        mercury.set_initial_timestep(1 | units.day)
        mercury.parameters.timestep = 1 | units.day
        
        mercury.particles.add_particles(solsys)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()

    def test5(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        self.assertEqual(mercury.get_name_of_current_state(), 'UNINITIALIZED')
        mercury.initialize_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'INITIALIZED')
        mercury.commit_parameters()
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')

        mercury.commit_particles()
        self.assertEqual(mercury.get_name_of_current_state(), 'RUN')

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')

        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.cleanup_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'END')
        mercury.stop()

    def test6(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        self.assertEqual(mercury.get_name_of_current_state(), 'UNINITIALIZED')
        mercury.initialize_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'INITIALIZED')
        mercury.commit_parameters()
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters[4:5])
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.commit_particles()
        self.assertEqual(mercury.get_name_of_current_state(), 'RUN')

        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        mercury.orbiters.add_particles(orbiters[0:1])
        self.assertEqual(mercury.get_name_of_current_state(), 'UPDATE')
        mercury.recommit_particles()

        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        mercury.cleanup_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'END')
        mercury.stop()

    def test7(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()#debugger="gdb")
        self.assertEqual(mercury.get_name_of_current_state(), 'UNINITIALIZED')
        mercury.initialize_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'INITIALIZED')
        mercury.commit_parameters()
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters[0:5])
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.commit_particles()
        self.assertEqual(mercury.get_name_of_current_state(), 'RUN')

        start_pos = mercury.orbiters[4].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[4].position, start_pos, 1)

        mercury.orbiters.remove_particles(orbiters[0:1])
        self.assertEqual(mercury.get_name_of_current_state(), 'UPDATE')
        mercury.recommit_particles()

        self.assertAlmostEqual(mercury.orbiters[3].position, start_pos, 1)

        start_pos = mercury.orbiters[3].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[3].position, start_pos, 1)

        mercury.cleanup_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'END')
        mercury.stop()


    def test8(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()#debugger="gdb")
        self.assertEqual(mercury.get_name_of_current_state(), 'UNINITIALIZED')
        mercury.initialize_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'INITIALIZED')
        mercury.commit_parameters()
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters[0:5])
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.commit_particles()
        self.assertEqual(mercury.get_name_of_current_state(), 'RUN')

        start_pos = mercury.orbiters[4].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[4].position, start_pos, 1)

        mercury.orbiters.remove_particles(orbiters[0:4])
        self.assertEqual(mercury.get_name_of_current_state(), 'UPDATE')
        mercury.recommit_particles()

        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        mercury.cleanup_code()
        self.assertEqual(mercury.get_name_of_current_state(), 'END')
        mercury.stop()

    def test9(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()#debugger="gdb")
        self.assertEqual(mercury.get_name_of_current_state(), 'UNINITIALIZED')
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters[0:5])
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        start_pos = mercury.orbiters[4].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[4].position, start_pos, 1)

        mercury.orbiters.remove_particles(orbiters[0:4])
        self.assertEqual(mercury.get_name_of_current_state(), 'UPDATE')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

    def test10(self):
        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        mercury.central_particle.add_particles(centre)
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        mercury.orbiters.add_particles(orbiters[4:5])
        self.assertEqual(mercury.get_name_of_current_state(), 'EDIT')
        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

        mercury.orbiters.add_particles(orbiters[0:4])
        self.assertEqual(mercury.get_name_of_current_state(), 'UPDATE')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)
        start_pos = mercury.orbiters[0].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertEqual(mercury.get_name_of_current_state(), 'EVOLVED')
        self.assertAlmostEqual(mercury.orbiters[0].position, start_pos, 1)

    def test11(self):
        solsys = new_solar_system()

        mercury = Mercury()
        self.assertEqual(mercury.parameters.timestep, 8 | units.day)
        mercury.parameters.timestep = 1 | units.day
        self.assertEqual(mercury.parameters.timestep, 1 | units.day)
        
        mercury.particles.add_particles(solsys)
        start_pos = mercury.particles[5].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertAlmostEqual(mercury.particles[5].position, start_pos, 1)
        mercury.stop()

    def test12(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.parameters.timestep = 1. | units.day
        
        mercury.particles.add_particles(solsys)
        start_pos = mercury.particles[5].position
        mercury.evolve_model(11.8618|units.yr)
        self.assertAlmostEqual(mercury.particles[5].position, start_pos, 1)
        mercury.particles.remove_particles(mercury.particles[1:5])
        self.assertAlmostEqual(mercury.particles[1].position, start_pos, 1)
        start_pos = mercury.particles[1].position
        mercury.evolve_model(2*11.8618|units.yr)
        self.assertAlmostEqual(mercury.particles[1].position, start_pos, 1)
        mercury.stop()

    def test13(self):
        solsys = new_solar_system()

        mercury = Mercury()        
        mercury.particles.add_particles(solsys)
        idpos1 = [ (p.position - q.position) for p in mercury.particles[1:10] for q in mercury.particles[1:10] ]
        mercury.evolve_model(11.8618|units.yr)
        edpos1 = [ (p.position - q.position) for p in mercury.particles[1:10] for q in mercury.particles[1:10] ]
        mercury.stop()

        centre, orbiters = new_solar_system_for_mercury()

        mercury = MercuryWayWard()
        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiters)

        idpos2 = [ (p.position - q.position) for p in mercury.orbiters[0:9] for q in mercury.orbiters[0:9] ]
        mercury.evolve_model(11.8618|units.yr)
        edpos2 = [ (p.position - q.position) for p in mercury.orbiters[0:9] for q in mercury.orbiters[0:9] ]
        mercury.stop()

        for d1,d2 in zip(idpos1,idpos2):
          self.assertAlmostEqual(d1,d2, 7)
        for d1,d2 in zip(edpos1,edpos2):
          self.assertAlmostEqual(d1,d2, 7)

    def test14(self):
        centre, orbiters = new_solar_system_for_mercury()
        oneyear = 365.14 | units.day
        halfyear = oneyear / 2.0
        mercury = MercuryWayWard()
        mercury.initialize_code()
        mercury.commit_parameters()

        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiters)

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(halfyear)
        central_particles = mercury.central_particle.copy()
        orbiters = mercury.orbiters.copy()
        mercury.stop()
        
        mercury = MercuryWayWard()
        mercury.initialize_code()
        mercury.parameters.begin_time = halfyear
        mercury.commit_parameters()

        mercury.central_particle.add_particles(centre)
        mercury.orbiters.add_particles(orbiters)
        mercury.evolve_model(oneyear)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()

    def test15(self):
        solsys = new_solar_system()

        solsys.x-=1.| units.AU

        p=datamodel.Particles(3)
        p.mass=[1,2,3] | units.MSun
        p.x=[1,10,100] | units.AU
        p.y=[0,0,-10] | units.AU
        p.z=[0,0,10] | units.AU

        pe=p.potential_energy_in_field(solsys)

        mercury = Mercury()
        mercury.particles.add_particles(solsys)
        pot=mercury.get_potential_at_point(p.x*0.,p.x,p.y,p.z)

        self.assertAlmostRelativeEqual((pot*p.mass).sum(),pe,12)

    def test16(self):
        solsys = new_solar_system()

        solsys.x-=1.| units.AU

        p=datamodel.Particles(3)
        p.mass=[1,2,3] | units.MSun
        p.x=[1,10,100] | units.AU
        p.y=[0,0,-10] | units.AU
        p.z=[0,0,10] | units.AU

        from amuse.community.huayno.interface import Huayno
        from amuse.units import nbody_system        
        conv=nbody_system.nbody_to_si(1. | units.AU, 1.| units.MSun)
        h = Huayno(conv)
        h.particles.add_particles(solsys)
        
        ax1,ay1,az1=h.get_gravity_at_point(p.x*0.,p.x,p.y,p.z)
        
        mercury = Mercury()        
        mercury.particles.add_particles(solsys)

        ax2,ay2,az2=mercury.get_gravity_at_point(p.x*0.,p.x,p.y,p.z)


        self.assertAlmostRelativeEqual(ax1,ax2,12)
        self.assertAlmostRelativeEqual(ay1,ay2,12)
        self.assertAlmostRelativeEqual(az1,az2,12)

    def test17(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.particles.add_particles(solsys)
        mercury.evolve_model(5.|units.yr)
        mercury.evolve_model(10.|units.yr)

        dpos1=mercury.particles[0].position-mercury.particles[5].position

        mercury.stop()

        mercury = Mercury()
        mercury.particles.add_particles(solsys)
        mercury.evolve_model(5.|units.yr)
        mercury.particles.x+=1 | units.AU
        mercury.particles.y+=2 | units.AU
        mercury.particles.z+=3 | units.AU
        mercury.particles.vx+=10. | units.kms
        mercury.particles.vy+=20. | units.kms
        mercury.particles.vz+=30. | units.kms
        mercury.evolve_model(10.|units.yr)

        dpos2=mercury.particles[0].position-mercury.particles[5].position
        mercury.stop()
                
        self.assertAlmostEqual(dpos1, dpos2, 12)

    def test18(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.initialize_code()
        self.assertEqual(mercury.parameters.integrator,10)
        mercury.parameters.integrator=2
        self.assertEqual(mercury.parameters.integrator,2)
        mercury.stop()

    def test19(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.initialize_code()
        mercury.parameters.integrator=1

        mercury.particles.add_particles(solsys)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()

    def test20(self):
        solsys = new_solar_system()

        mercury = Mercury()#debugger="gdb")
        mercury.initialize_code()
        mercury.parameters.integrator=2

        mercury.particles.add_particles(solsys)
        mercury.commit_particles()

        start_pos = mercury.orbiters[2].position
        mercury.evolve_model(365.14|units.day)
        self.assertAlmostEqual(mercury.orbiters[2].position, start_pos, 1)
        mercury.stop()

    def test21(self):
        solsys = new_solar_system()

        mercury = Mercury()
        mercury.initialize_code()
        
        names=["elements_file","close_encounters_file","info_file",
         "bigbody_file","smallbody_file","integration_parameters_file","restart_file"]
        
        for name in names:
          self.assertEqual(getattr(mercury.parameters,name),"/dev/null")
        
        for name in names:
          setattr(mercury.parameters,name,os.path.join(mercury.output_directory,name))

        for name in names:
          self.assertEqual(getattr(mercury.parameters,name),os.path.join(mercury.output_directory,name))

        mercury.stop()

    def xtest22(self):
        """ collision test hangs or fails if internal collision detection is enabled """
         
        def collision():
          
            M=1.| units.MSun
            m=1.| units.MJupiter
            r=5. | units.AU
            vcirc=(constants.G*(M+m)/r)**0.5
            
            sys=datamodel.Particles(4)
            sys[0].mass=M
            sys[0].radius=1. | units.RSun
            sys[0].x=0 | units.AU
            sys[0].y=0 | units.AU
            sys[0].z=0 | units.AU
            sys[0].vx=0 | units.kms
            sys[0].vy=0 | units.kms
            sys[0].vz=0 | units.kms
            
            sys[1].mass=m
            sys[1].radius=0.01 | units.RSun
            sys[1].x=r
            sys[1].y=0 | units.AU
            sys[1].z=0 | units.AU
            sys[1].vx=0 | units.kms
            sys[1].vy=vcirc
            sys[1].vz=0 | units.kms
            
            sys[2].mass=m
            sys[2].radius=0.01 | units.RSun
            sys[2].x=-r
            sys[2].y=0 | units.AU
            sys[2].z=0 | units.AU
            sys[2].vx=0 | units.kms
            sys[2].vy=vcirc
            sys[2].vz=0 | units.kms
          
            sys[3].mass=m
            sys[3].radius=0.01 | units.RSun
            sys[3].x=0 | units.AU
            sys[3].y=r
            sys[3].z=0 | units.AU
            sys[3].vx=0 | units.kms
            sys[3].vy=0 | units.kms
            sys[3].vz=0 | units.kms
            
            return sys
  
        code=Mercury()
                
        sys=collision()
        
        code.particles.add_particles(sys)
        
        tend=3.5| units.yr
        dt=100. | units.day
        tnow=code.model_time
                
        while tnow<tend:
          code.evolve_model(tnow+dt)
          tnow=code.model_time
          print(tnow.in_(units.yr))

        code.stop()
        



from amuse.test import amusetest

from amuse.support.interface import InCodeComponentImplementation

from amuse.datamodel.indexing import *

from amuse.datamodel.grids import *

import numpy
import inspect
import collections
from amuse.units import units
from amuse.units import constants
from amuse.units import nbody_system
from amuse import datamodel

class TestGrids(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel.Grid(5,4,3)
        grid.mass = 2.0 | units.kg
        self.assertEqual(grid.mass[0][1][2], 2.0 | units.kg)
        self.assertEqual(grid[0][1][2].mass, 2.0 | units.kg)
        self.assertEqual(len(grid.mass), 5)
        
    def test2(self):
        grid = datamodel.Grid(5,4,3)
        grid.mass = units.kg.new_quantity(numpy.arange(5*4*3).reshape(5,4,3))
        self.assertEqual(grid.number_of_dimensions(), 3)
        subgrid = grid[1]
        self.assertEqual(subgrid.number_of_dimensions(), 2)
        self.assertEqual(subgrid.mass.number.shape, (4,3))
    
    def test3(self):
        grid = datamodel.Grid(5,4,3)
        values = numpy.arange(5*4*3).reshape(5,4,3)
        grid.mass = units.kg.new_quantity(values)
        self.assertEqual(grid.number_of_dimensions(), 3)
       
        subgrid = grid[1][2]
        self.assertEqual(subgrid.number_of_dimensions(), 1)
        self.assertEqual(subgrid.mass.number.shape, (3,))
        self.assertTrue(numpy.all(values[1][2] == subgrid.mass.value_in(units.kg)))
        
    def test4(self):
        grid = datamodel.Grid(5,4,3)
        values = numpy.arange(5*4*3).reshape(5,4,3)
        grid.mass = units.kg.new_quantity(values)
        self.assertEqual(grid.number_of_dimensions(), 3)
       
        gridpoint = grid[1][2][1]
        self.assertEqual(gridpoint.mass, 19.0 | units.kg)
        gridpoint = grid[1][2][2]
        self.assertEqual(gridpoint.mass, 20.0 | units.kg)
        
    def test5(self):
        grid = datamodel.Grid(5,4,3)
        grid.add_calculated_attribute("squared_mass", lambda m : m * m, ["mass",])
        values = numpy.arange(5*4*3).reshape(5,4,3)
        grid.mass = units.kg.new_quantity(values)
        gridpoint = grid[1][2][1]
        self.assertEqual(gridpoint.mass, 19.0 | units.kg)
        self.assertEqual(gridpoint.squared_mass, (19.0 | units.kg) ** 2)
        subgrid = grid[1][2]
        self.assertTrue(numpy.all(subgrid.squared_mass == ([18.0, 19.0, 20.0] | units.kg) ** 2))
        
    def test6(self):
        grid = datamodel.Grid(5,4,3)
        grid.add_function_attribute("sum_mass", lambda grid, x : grid.mass.sum() + x, lambda grid, gridpoint, x : gridpoint.mass + x)
        values = numpy.arange(5*4*3).reshape(5,4,3)
        grid.mass = units.kg.new_quantity(values)
        gridpoint = grid[1][2][1]
        self.assertEqual(gridpoint.mass, 19.0 | units.kg)
        self.assertEqual(gridpoint.sum_mass(2.0 | units.kg), (21.0 | units.kg) )
        subgrid = grid[1][2]
        self.assertTrue(numpy.all(subgrid.sum_mass(2 | units.kg) == (18.0 + 19.0 + 20.0 + 2.0 | units.kg)))
        
    def test7(self):
        grid = datamodel.Grid(5,4,3)
        grid.add_vector_attribute("position", ["x","y","z"])
        x = numpy.arange(5*4*3).reshape(5,4,3)
        y = x + 100.0
        z = x + 200.0
        grid.x = units.m.new_quantity(x)
        grid.y = units.m.new_quantity(y)
        grid.z = units.m.new_quantity(z)
        gridpoint = grid[1][2][1]
        self.assertEqual(gridpoint.position[0], 19 | units.m)
        self.assertEqual(gridpoint.position[1], 119 | units.m)
        self.assertEqual(gridpoint.position[2], 219 | units.m)
        subgrid = grid[1][2]
        self.assertEqual(subgrid.position[1][0], 19 | units.m)
        self.assertEqual(subgrid.position[1][1], 119 | units.m)
        self.assertEqual(subgrid.position[1][2], 219 | units.m)
        
    
    def test8(self):
        grid0 = datamodel.Grid(5,4,3)
        x = numpy.arange(5*4*3).reshape(5,4,3)
        y = x + 100.0
        grid0.x = units.m.new_quantity(x)
        grid0.y = units.m.new_quantity(y)
        
        grid1 = datamodel.Grid(5,4,3)
        x = numpy.arange(5*4*3).reshape(5,4,3)
        x = x + 200.0
        y = x + 200.0
        grid1.x = units.m.new_quantity(x)
        grid1.y = units.m.new_quantity(y)
    
        self.assertTrue(numpy.all(grid0[1][2].x != grid1[1][2].x))
        
        channel = grid0.new_channel_to(grid1)
        channel.copy_attributes(["x",])
        
        self.assertTrue(numpy.all(grid0[1][2].x == grid1[1][2].x))
        self.assertTrue(numpy.all(grid0[1][2].y != grid1[1][2].y))
        

        

    def test9(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[0][0][0].x, 0.1 | units.m)
        self.assertEqual(grid[0][0][0].y, 0.125 | units.m)
        self.assertEqual(grid[0][0][0].z, 0.25 | units.m)
        self.assertEqual(grid[...,0,0].x, [0.1,0.3,0.5,0.7,0.9] | units.m)
        self.assertEqual(grid[0,0,...].z, [0.25, 0.75] | units.m)
        
        cellsize = grid.cellsize()
        self.assertAlmostRelativeEquals(cellsize[0], 0.2 | units.m)
        self.assertAlmostRelativeEquals(cellsize[1], 0.25 | units.m)
        self.assertAlmostRelativeEquals(cellsize[2], 0.5 | units.m)
    
    

    def test11(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        iarray,jarray,karray = grid.indices()
        for i in range(5):
            for j in range(4):
                for k in range(2):
                    self.assertEqual(iarray[i][j][k], i)
                    self.assertEqual(jarray[i][j][k], j)
                    self.assertEqual(karray[i][j][k], k)
        iarray,jarray,karray = grid.indices()
        i = 0
        for j in range(4):
            for k in range(2):
                self.assertEqual(iarray[i][j][k], i)
                self.assertEqual(jarray[i][j][k], j)
                self.assertEqual(karray[i][j][k], k)
       
        iarray,jarray,karray = grid[...,0,0].indices()
        j = 0
        k = 0
        for i in range(5):
            self.assertEqual(iarray[i], i)
            self.assertEqual(jarray[i], j)
            self.assertEqual(karray[i], k)
        iarray,jarray,karray = grid[3,2,...].indices()
        i = 3
        j = 2
        for k in range(2):
            self.assertEqual(iarray[k], i)
            self.assertEqual(jarray[k], j)
            self.assertEqual(karray[k], k)
        iarray,jarray,karray = grid[2,...,1].indices()
        i = 2
        k = 1
        for j in range(4):
            self.assertEqual(iarray[j], i)
            self.assertEqual(jarray[j], j)
            self.assertEqual(karray[j], k)
    
    
    def test12(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[grid.x > 0.4| units.m].x.shape, (24,))
        self.assertEqual(grid[grid.x>0.4| units.m].x, grid.x[grid.x>0.4|units.m])
        iarray,jarray,karray = grid.indices()
        self.assertEqual(grid[grid.x>0.4| units.m].indices()[0],  iarray[grid.x>0.4| units.m])
        self.assertEqual(grid[grid.x>0.4| units.m].indices()[1],  jarray[grid.x>0.4| units.m])
        self.assertEqual(grid[grid.x>0.4| units.m].indices()[2],  karray[grid.x>0.4| units.m])  
        
    def test13(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[0].shape, (4,2))
        self.assertEqual(grid[0][0].shape, (2,))
        self.assertEqual(grid[...,2,1].shape, (5,))
        
    def test14(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[0].x, grid.x[0])
        self.assertEqual(grid[0][1].x, grid.x[0][1])
        self.assertEqual(grid[1][2][1].x, grid.x[1][2][1])
        self.assertEqual(grid[...,2,1].x, grid.x[...,2,1])
        self.assertEqual(grid[1,...,1].x, grid.x[1,...,1])
        self.assertEqual(grid[1,2,...].x, grid.x[1,2,...])
        self.assertEqual(grid[...,1].x, grid.x[...,1])
        self.assertEqual(grid[2,...].x, grid.x[2,...])
        self.assertEqual(grid[:,3,:].x, grid.x[:,3,:])
        self.assertEqual(grid[:,3,:].y, grid.y[:,3,:])
        self.assertEqual(grid[:,3,:].z, grid.z[:,3,:])
    
    def test15(self):
        
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        nk = nj = ni =0
        for plane1 in grid:
            nk += 1
            for plane2 in plane1:
                nj += 1
                for plane3 in plane2:
                    ni += 1
        self.assertEqual(nk, 5)
        self.assertEqual(nj, 4 * 5)
        self.assertEqual(ni, 2 * 4 * 5)
                    
    def test16(self):
        grid = datamodel.new_regular_grid((5,4,2,4), [1.0 | units.m, 1.0 | units.m, 1.0 | units.m, 1.0 | units.s], ('x', 'y', 'z', 't') )
        self.assertEqual(grid.shape, (5,4,2,4))
        self.assertEqual(grid.x.shape, (5,4,2,4))
        self.assertAlmostRelativeEquals( grid[1][2][1].x, ([0.3] * 4) | units.m)
        self.assertAlmostRelativeEquals( grid[1][2][1].t, [0.125, 0.375, 0.625, 0.875] | units.s)
        self.assertEqual(grid[0].x, grid.x[0])
        self.assertEqual(grid[0][1].x, grid.x[0][1])
        self.assertEqual(grid[1][2][1].x, grid.x[1][2][1])
        self.assertEqual(grid[1][2][1][2].x, grid.x[1][2][1][2])

    def test17(self):
        grid = datamodel.new_regular_grid((4,2), [1.0 | units.m, 1.0 | units.m])
        self.assertEqual(grid.shape, (4,2) )
        self.assertEqual(grid.x.shape, (4,2))
        self.assertAlmostRelativeEquals( grid[1].x, ([0.375] * 2) | units.m)
        self.assertAlmostRelativeEquals( grid[1][1].y, 0.75 | units.m)
        self.assertEqual(grid[0].x, grid.x[0])
        self.assertEqual(grid[0][1].x, grid.x[0][1])
        
    def test18(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid.shape, (5,4,2) )
        self.assertEqual(grid[1:2,...].x.shape, (1,4,2 ))
        self.assertEqual(grid[1:2,...].shape, (1,4,2) )
        self.assertEqual(grid[1:2,...].x, grid.x[1:2,...])
        self.assertEqual(grid[1:3,...].x.shape, (2,4,2) )
        self.assertEqual(grid[1:3,...].shape, (2,4,2) )
        self.assertEqual(grid[1:3,...].x, grid.x[1:3,...])

    def test19(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[1:3,...].x, grid.x[1:3,...])
        self.assertEqual(grid[1:3,2:3,...].x, grid.x[1:3,2:3,...])
        self.assertEqual(grid[1:3,2:3,0:1].x, grid.x[1:3,2:3,0:1])
        self.assertEqual(grid[1:3,...,0:1].x, grid.x[1:3,...,0:1])
        self.assertEqual(grid[...,0:1].x, grid.x[...,0:1])
        self.assertEqual(grid[...,2:3,0:1].x, grid.x[...,2:3,0:1])
        
    def test20(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[1:3,:,:].x, grid.x[1:3,:,:])
        self.assertEqual(grid[1:3,2:3,:].x, grid.x[1:3,2:3,:])
        self.assertEqual(grid[1:3,2:3,0:1].x, grid.x[1:3,2:3,0:1])
        self.assertEqual(grid[1:3,:,0:1].x, grid.x[1:3,:,0:1])
        self.assertEqual(grid[:,:,0:1].x, grid.x[:,:,0:1])
        self.assertEqual(grid[:,2:3,0:1].x, grid.x[:,2:3,0:1])
        
        
    def test21(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid[1:3,:,:].copy().x, grid.x[1:3,:,:])
        self.assertEqual(grid[1:3,:,:].copy().shape, (2,4,2))
        self.assertEqual(grid[1:3,2:3,:].copy().x, grid.x[1:3,2:3,:])
        self.assertEqual(grid[1:3,2:3,:].copy().shape, (2,1,2))
        self.assertEqual(grid[1:3,2:3,0:1].copy().x, grid.x[1:3,2:3,0:1])
        self.assertEqual(grid[1:3,2:3,0:1].copy().shape, (2,1,1))
        self.assertEqual(grid[1:3,:,0:1].copy().x, grid.x[1:3,:,0:1])
        self.assertEqual(grid[1:3,:,0:1].copy().shape, (2,4,1))
        self.assertEqual(grid[:,:,0:1].copy().x, grid.x[:,:,0:1])
        self.assertEqual(grid[:,:,0:1].copy().shape, (5,4,1))
        self.assertEqual(grid[:,2:3,0:1].copy().x, grid.x[:,2:3,0:1])
        self.assertEqual(grid[:,2:3,0:1].copy().shape, (5,1,1))
        
    
    def test22(self):
        grid1 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        slice1 = grid1[1:3,:,:].copy()
        slice2 = grid2[1:3,:,:]
        slice1.x = -10 | units.m
        channel = slice1.new_channel_to(slice2)
        channel.copy()
        self.assertEqual(grid2.x[1:3], -10 | units.m)
        self.assertEqual(grid2.x[4],grid1.x[4])
    
    
    def test23(self):
        grid1 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual(grid1[1:3,...].shape,(2,4,2))
        slice1 = grid1[1:3,...].copy()
        slice2 = grid2[1:3,...]
        slice1.x = -10 | units.m
        channel = slice1.new_channel_to(slice2)
        channel.copy()
        self.assertEqual(grid2.x[1:3], -10 | units.m)
        self.assertEqual(grid2.x[4],grid1.x[4])
        
    
    def test24(self):
        particle = datamodel.Particle()
        particle.mass = 10 | units.kg
        
        grid = datamodel.Grid(5,4,3)
        grid.mass = 2.0 | units.kg
        grid.nounit = 10
        self.assertEqual(grid.nounit[0][1][2], 10)
        self.assertEqual(grid[0][1][2].nounit, 10)
        self.assertEqual(len(grid.nounit), 5)
        #grid[0][1][0].particle = particle
        #self.assertEquals(grid.mass[0][1][2], 2.0 | units.kg)
        #self.assertEquals(grid[0][1][0].particle, particle)
        #self.assertEquals(grid[0][1][1].particle, None)


    def test25(self):
        grid = datamodel.Grid(5,4,3)
        grid.mass = 2.0 | units.kg
        for cell in grid.iter_cells():
            self.assertEqual(cell.mass, 2.0 | units.kg)
    
    def test26(self):
        
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
    
        xv, yv, zv = numpy.mgrid[0:5,0:4,0:2]
        xv = xv.flatten()
        yv = yv.flatten()
        zv = zv.flatten()
        i = 0
        for cell in grid.iter_cells():
            expected_position = grid[xv[i], yv[i], zv[i]].position
            self.assertEqual(cell.position, expected_position)
            i += 1

    def test27(self):
        grid = datamodel.new_regular_grid((3,3), [1.0, 1.0] | units.m)
        subgrid1=grid[0:1,0:2]
        subgrid2=grid[0:1,0:2]
        subgrid3=grid[0:2,0:3][0:1,0:2]
        subgrid4=grid[0:1,0:3]
                
        self.assertTrue(subgrid1==subgrid2)
        self.assertTrue(subgrid1==subgrid3)
        self.assertTrue(subgrid2==subgrid3)
        self.assertFalse(subgrid1==subgrid4)
        self.assertFalse(subgrid2==subgrid4)
        self.assertFalse(subgrid3==subgrid4)

    def test28(self):
        grid = datamodel.Grid(200,400)
        subgrid1=grid[1:-1,1:-1]
        subgrid2=subgrid1[3:5,1:399]
        self.assertEqual(subgrid2.shape,(2,397))

    def test29(self):
        grid = datamodel.Grid(200)
        subgrid1=grid[1:-1]
        subgrid2=subgrid1[3:5]
        self.assertEqual(subgrid2.shape,(2,))

    def test30(self):
        grid = datamodel.Grid(200)
        subgrid1=grid[1:199]
        subgrid2=subgrid1[3:5]
        self.assertEqual(subgrid2.shape,(2,))

    def test31(self):
        grid = datamodel.Grid(20,20,20)
        a=numpy.zeros((20,20,20))
        self.assertEqual(a[1].shape,grid[1].shape) 
        self.assertEqual(a[1,2].shape,grid[1,2].shape) 
        self.assertEqual(a[1:5].shape,grid[1:5].shape) 
        self.assertEqual(a[2,1:5].shape,grid[2,1:5].shape)
        self.assertEqual(a[2,...].shape,grid[2,...].shape)       
        self.assertEqual(a[...,3,:].shape,grid[...,3,:].shape)       
        self.assertEqual(a[...,3:5,:].shape,grid[...,3:5,:].shape)       
        self.assertEqual(a[...,3:5,:].shape,grid[...,3:5,:].shape)
        self.assertEqual(a[::,::2].shape,grid[::,::2].shape)        

    def test32(self):
        grid = datamodel.Grid(200)
        grid.mass=numpy.arange(200)

        self.assertEqual(grid[1].mass,grid.mass[1]) 
        self.assertEqual(grid[1:-1].mass,grid.mass[1:-1])
        self.assertEqual(grid[-1:1:-1].mass,grid.mass[-1:1:-1])
        self.assertEqual(grid[-1:1:-1][1:-1].mass,grid.mass[-1:1:-1][1:-1])
        self.assertEqual(grid[-1:1:-1][-1:1].mass,grid.mass[-1:1:-1][-1:1])
        self.assertEqual(grid[-1:1:-1][-1:1:-3].mass,grid.mass[-1:1:-1][-1:1:-3])
        self.assertEqual(grid[300:1:-2][-1:5:-3].mass,grid.mass[300:1:-2][-1:5:-3])
        self.assertEqual(grid[100::-2][::3].mass,grid.mass[100::-2][::3])
        self.assertEqual(grid[100::-2][::-3].mass,grid.mass[100::-2][::-3])


    def test32b(self):
        grid = datamodel.Grid(200)
        grid.mass=numpy.arange(200)

        self.assertEqual(grid[::-1].mass,grid.mass[::-1])
        self.assertEqual(grid[10::-1].mass,grid.mass[10::-1])
        self.assertEqual(grid[:100:2].mass,grid.mass[:100:2])        
        self.assertEqual(grid[-1::-1].mass,grid.mass[-1::-1])
        self.assertEqual(grid[-1:-300:-1].mass,grid.mass[-1:-300:-1])
        self.assertEqual(grid[300:-300:-1].mass,grid.mass[300:-300:-1])        
        self.assertEqual(grid[300:-300:-7].mass,grid.mass[300:-300:-7])        
        
        
        
    def test33(self):
        grid = datamodel.Grid(20)
        grid.mass=numpy.zeros((20,5))
        self.assertEqual(grid[1].mass,numpy.zeros(5))
        grid.mass=numpy.arange(5)
        self.assertEqual(grid[-1].mass,numpy.arange(5))
        subgrid=grid[::2]
        self.assertEqual(subgrid[-1].mass,numpy.arange(5))
        subgrid[1].mass=5-numpy.arange(5)
        self.assertEqual(subgrid[1].mass,5-numpy.arange(5))
        self.assertEqual(grid[2].mass,5-numpy.arange(5))

    def test34(self):
        grid = datamodel.Grid(20)
        grid.mass=numpy.zeros((20,5)) | units.kg
        self.assertEqual(grid[1].mass,numpy.zeros(5) | units.kg)
        grid.mass=numpy.arange(5) | units.kg
        self.assertEqual(grid[-1].mass,numpy.arange(5)| units.kg)
        subgrid=grid[::2]
        self.assertEqual(subgrid[-1].mass,numpy.arange(5)| units.kg)
        subgrid[1].mass=(5-numpy.arange(5))| units.kg
        self.assertEqual(subgrid[1].mass,(5-numpy.arange(5))| units.kg)
        self.assertEqual(grid[2].mass,(5-numpy.arange(5))| units.kg)

    def test35(self):
        grid=datamodel.Grid(10,5)
        grid.mass=numpy.zeros((10,5,3))
        self.assertEqual(grid[2,2].mass,numpy.zeros(3))
        grid[::3,::2].mass=numpy.arange(3)
        self.assertEqual(grid[3,2].mass,numpy.arange(3))

    def test36(self):
        grid=datamodel.Grid(10)
        grid.mass=numpy.zeros((10,5,3))
        self.assertEqual(grid[2].mass,numpy.zeros((5,3)))
        grid[::3].mass=numpy.ones((5,3))
        self.assertEqual(grid[3].mass,numpy.ones((5,3)))

    def test37(self):
        grid = datamodel.Grid(20)
        grid.mass=numpy.zeros((20,5))
        grid.mass=numpy.arange(5)
        self.assertEqual(grid[-1].mass,numpy.arange(5))
        self.assertEqual(grid.mass.shape,(20,5))
        subgrid=grid[::2]
        self.assertEqual(subgrid[-1].mass,numpy.arange(5))
        subgrid[1].mass=5-numpy.arange(5)
        self.assertEqual(subgrid[1].mass,5-numpy.arange(5))
        self.assertEqual(grid[2].mass,5-numpy.arange(5))
        cp=subgrid.copy()
        self.assertEqual(cp[1].mass,5-numpy.arange(5))
        self.assertEqual(cp.mass.shape,(10,5))       
        cp=grid.copy()
        self.assertEqual(cp.mass.shape,(20,5))
        self.assertEqual(cp[2].mass,5-numpy.arange(5))
        self.assertEqual(cp[-1].mass,numpy.arange(5))

    def test38(self):
        grid=datamodel.new_cartesian_grid((10,),1)
        sub=grid[::2]
        self.assertEqual(sub[0].x,0.5)
        self.assertEqual(sub[(0,)].x,0.5)

        grid=datamodel.new_cartesian_grid((10,10),1)
        sub=grid[::2,::]
        self.assertEqual(sub[0,0].x,0.5)
        self.assertEqual(sub[(0,1)].y,1.5)

    def test39(self):
        grid=datamodel.new_cartesian_grid((10,10),1)
        sub=grid[3:6,5:8]
        self.assertEqual(sub[0:-1,0:-1].x,sub.x[0:-1,0:-1])
        self.assertEqual(sub[0:-1,-1].x,sub.x[0:-1,-1])
        self.assertEqual(sub[-1,-1].x,sub.x[-1,-1])
        self.assertEqual(sub[-1,-2].x,sub.x[-1,-2])

    def test40(self):
        grid1 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        grid1.m1 = 1
        grid1.m2 = 2
        channel = grid1.new_channel_to(grid2)
        channel.transform(["m3"],lambda x,y: (x,),["m1","m2"])
        self.assertEqual(grid2.m3, 1)
        channel.transform(["m3"],lambda x,y: (y,),["m1","m2"])
        self.assertEqual(grid2.m3, 2)
        channel.transform(["m3"],lambda x,y: (x+y,),["m1","m2"])
        self.assertEqual(grid2.m3, 3)
        channel.transform(["m3","m4"],lambda x,y: (x+y,2*x-y),["m1","m2"])
        self.assertEqual(grid2.m3, 3)
        self.assertEqual(grid2.m4, 0)

    def test40b(self):
        grid = datamodel.new_regular_grid((50,), [1.0] | units.m)
        for index in [ [0], [0,3,4], [1,2,2],[[2,3]],[[0,1],[2,3]],list(range(50)) ]:
          i=numpy.array(index)
          self.assertEqual(grid[i].x, grid.x[ i ])
          self.assertEqual(grid[i][1:].x, grid.x[ i ][1:])
          self.assertEqual(grid[i][1::2].x, grid.x[ i ][1::2])

    def test41(self):
        grid = datamodel.new_regular_grid((10,10), [1.0,2.0] | units.m)
        for _i,_j in [ ([0,1],[2,3]) ]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(grid[i,j].x, grid.x[ i,j ])
          self.assertEqual(grid[i,j][1:].x, grid.x[ i,j ][1:])
          self.assertEqual(grid[i,j][1::2].x, grid.x[ i,j ][1::2])

    def test42(self):
        grid = datamodel.new_regular_grid((3,4,5,6), [1.0,2.0,3.0,4.0],axes_names="abcd")
        for _i,_j in [ ([0],[3]),([0,1],[2,3]) ]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(grid[i,j].a, grid.a[ i,j ])
          self.assertEqual(grid[0,1,i,j].a, grid.a[0,1, i,j ])
          self.assertEqual(grid[i,j,0,0].a, grid.a[ i,j,0,0 ])
          self.assertEqual(grid[i,1,2,j].a, grid.a[ i,1,2,j])

    def test43(self):
        grid = datamodel.new_regular_grid((3,4,5,6), [1.0,2.0,3.0,4.0],axes_names="abcd")
        for _i,_j in [ ([0,1],[2,3]) ]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          subgrid=grid[:,0,:,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])
          subgrid=grid[:,0,2,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])
          subgrid=grid[:,0,-2,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])

    def test44(self):
        grid = datamodel.new_regular_grid((3,4,5,6), [1.0,2.0,3.0,4.0],axes_names="abcd")
        for _i,_j in [ ([-2],[-4]),([0,-1],[-2,3]) ]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          subgrid=grid[:,0,:,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])
          subgrid=grid[:,0,2,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])
          subgrid=grid[:,0,-2,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])

    def test45(self):
        grid = datamodel.new_regular_grid((3,4,5,6), [1.0,2.0,3.0,4.0],axes_names="abcd")
        for _i,_j in [ ([-1],[-4]) ]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          subgrid=grid[::2,0,:,:]
          self.assertEqual(subgrid[i,j].a, subgrid.a[ i,j ])

    def test46(self):
        grid = datamodel.new_regular_grid((6,), [1.0],axes_names="abcd")
        subgrid=grid[::2]
        self.assertEqual(subgrid[-2].a, subgrid.a[ -2 ])

        grid = datamodel.new_regular_grid((7,), [1.0],axes_names="abcd")
        subgrid=grid[::2]
        self.assertEqual(subgrid[-2].a, subgrid.a[ -2 ])

        grid = datamodel.new_regular_grid((7,), [1.0],axes_names="abcd")
        subgrid=grid[6:0:-2]
        self.assertEqual(subgrid[-2].a, subgrid.a[ -2 ])

    def test47(self):
        grid = datamodel.Grid()
        grid.mass=12.
        self.assertEqual(grid[...].mass,12.)
        self.assertEqual(grid.mass,12.)

    def test48(self):
        p=datamodel.Grid(3)

        p.a1=1.
        p.a2=1. | units.rad
        p.a3=1. | units.deg
        
        # test all combinations:
        
        p[0].a1=2.
        p[0].a2=2.
        p[0].a3=2.
                
        p[1].a1=2. | units.rad
        p[1].a2=2. | units.rad
        p[1].a3=2. | units.rad
        
        p[2].a1=2. | units.deg
        p[2].a2=2. | units.deg
        p[2].a3=2. | units.deg
        
        self.assertEqual( p.a1, [2.,2., (2. | units.deg).value_in(units.none)])
        self.assertEqual( p.a2, [2.,2., (2. | units.deg).value_in(units.none)])
        self.assertEqual( p.a3, [(2. | units.rad).in_(units.deg),
                                  (2. | units.rad).in_(units.deg) , 2. | units.deg])


    def test50(self):
        grid = datamodel.Grid(3,2)
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            grid.savepoint((i + 1) * 1.0 | units.s)

        dens = grid.get_timeline_of_attribute("density")
        self.assertEqual(len(dens), 3)
        self.assertEqual(dens[0][1].shape, (3,2))
        self.assertEqual(dens[1][1].shape, (3,2))
        self.assertEqual(dens[2][1].shape, (3,2))
        self.assertEqual(dens[0][0], 1.0 | units.s)
        self.assertEqual(dens[1][0], 2.0 | units.s)
        self.assertEqual(dens[2][0], 3.0 | units.s)

    def test51(self):
        grid = datamodel.Grid(3,2)
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            grid.savepoint((i + 1) * 1.0 | units.s)

        dens = grid[0:1,:].get_timeline_of_attribute("density")
        self.assertEqual(len(dens), 3)
        self.assertEqual(dens[0][1].shape, (1,2))
        self.assertEqual(dens[1][1].shape, (1,2))
        self.assertEqual(dens[2][1].shape, (1,2))
        self.assertEqual(dens[0][0], 1.0 | units.s)
        self.assertEqual(dens[1][0], 2.0 | units.s)
        self.assertEqual(dens[2][0], 3.0 | units.s)

    def test51(self):
        grid = datamodel.Grid(3,2)
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            grid.savepoint((i + 1) * 1.0 | units.s)

        time,dens = grid[0:1,:].get_timeline_of_attribute_as_vector("density")
        self.assertEqual(dens.shape, (3,1,2))
        self.assertEqual(dens[0].shape, (1,2))
        self.assertEqual(dens[1].shape, (1,2))
        self.assertEqual(dens[2].shape, (1,2))
        self.assertEqual(time[0], 1.0 | units.s)
        self.assertEqual(time[1], 2.0 | units.s)
        self.assertEqual(time[2], 3.0 | units.s)

    def test52(self):
        grid = datamodel.Grid(3,2)
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            grid.savepoint((i + 1) * 1.0 | units.s)

        time,dens = grid[2,1].get_timeline_of_attribute_as_vector("density")
        self.assertEqual(dens.shape, (3,))
        self.assertEqual(dens[0], 0 | units.kg/units.m**3)
        self.assertEqual(dens[1], 1 | units.kg/units.m**3)
        self.assertEqual(dens[2], 2 | units.kg/units.m**3)
        self.assertEqual(time[0], 1.0 | units.s)
        self.assertEqual(time[1], 2.0 | units.s)
        self.assertEqual(time[2], 3.0 | units.s)

    def test53(self):
        grid = datamodel.Grid(3,2)
        subgrid=grid[0:1,:]
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            subgrid.savepoint((i + 1) * 1.0 | units.s)

        time,dens = subgrid.get_timeline_of_attribute_as_vector("density")
        self.assertEqual(dens.shape, (3,1,2))
        self.assertEqual(dens[0], 0 | units.kg/units.m**3)
        self.assertEqual(dens[1], 1 | units.kg/units.m**3)
        self.assertEqual(dens[2], 2 | units.kg/units.m**3)
        self.assertEqual(time[0], 1.0 | units.s)
        self.assertEqual(time[1], 2.0 | units.s)
        self.assertEqual(time[2], 3.0 | units.s)

    def test54(self):
        """
        illustrates getting subgrid/gridpoint with history from subgrid with history
        """
        grid = datamodel.Grid(3,2)
        subgrid=grid[0:1,:]
        for i in range(3):
            grid.density = (i * 1.0) | units.kg/units.m**3
            subgrid.savepoint((i + 1) * 1.0 | units.s)
        # if the gridpoint derives directly from subgrid, its defined on the original
        # grid (which has no history...)
        subsub=subgrid.savepoint((i + 1) * 1.0 | units.s)
        time,dens = subsub[0,1].get_timeline_of_attribute_as_vector("density")
        self.assertEqual(dens.shape, (3,))
        self.assertEqual(dens[0], 0 | units.kg/units.m**3)
        self.assertEqual(dens[1], 1 | units.kg/units.m**3)
        self.assertEqual(dens[2], 2 | units.kg/units.m**3)
        self.assertEqual(time[0], 1.0 | units.s)
        self.assertEqual(time[1], 2.0 | units.s)
        self.assertEqual(time[2], 3.0 | units.s)

class TestGridFactories(amusetest.TestCase):
    def test1(self):
        grid1 = datamodel.new_cartesian_grid( (4,5), 1.0 | units.m)
        grid2 = datamodel.new_regular_grid( (4,5), [4.0,5.0] | units.m)
        grid3 = datamodel.new_rectilinear_grid( (4,5), [numpy.arange(5.) | units.m,numpy.arange(6.) | units.m])
        
        self.assertEqual(grid1.position,grid2.position)
        self.assertEqual(grid2.position,grid3.position)

    def test2(self):
        grid=datamodel.new_rectilinear_grid((10,),(1.*numpy.arange(11),))
        self.assertEqual(grid._axes_cell_boundaries,1.*numpy.arange(11))
        grid=datamodel.new_regular_grid((10,),[10.])
        self.assertEqual(grid._lengths,[10.])
        grid=datamodel.new_cartesian_grid((10,),1.)
        self.assertEqual(grid._cellsize,1.)
        grid=datamodel.new_regular_grid((10,20,),[10.,15.])
        self.assertEqual(grid._lengths,[10.,15.])

    def test3(self):
        N=10
        x,y=numpy.indices((N+1,N+1))
        grid=datamodel.new_structured_grid((N,N),[x,y])
        self.assertEqual(grid.shape,(N,N))
        x,y=numpy.indices((N,N))
        x=x+0.5
        y=y+0.5
        self.assertEqual(grid.x,x)
        self.assertEqual(grid.y,y)
        
    def test4(self):
        N=2
        x,y,z=numpy.indices((N+1,N+1,2*N+1))
        grid=datamodel.new_structured_grid((N,N,2*N),[x,y,z])
        self.assertEqual(grid.shape,(N,N,2*N))
        x,y,z=numpy.indices((N,N,2*N))
        x=x+0.5
        y=y+0.5
        z=z+0.5
        self.assertEqual(grid.x,x)
        self.assertEqual(grid.y,y)
        self.assertEqual(grid.z,z)



class TestGridAttributes(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertAlmostRelativeEquals(grid.get_minimum_position(),  ([0.0, 0.0, 0.0] | units.m) )
        self.assertAlmostRelativeEquals(grid.get_maximum_position(),  [1.0, 1.0, 1.0] | units.m)
        self.assertAlmostRelativeEquals(grid.get_volume(),  1.0 | units.m ** 3)
        self.assertTrue(grid.contains([0.5,0.5,0.5] | units.m))
        
    def test2(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertTrue(numpy.all(grid.contains([[0.5,0.5,0.5] , [0.1,0.1,0.1]]| units.m)))
        self.assertFalse(numpy.all(grid.contains([[1.1,0.5,0.5] , [0.1,1.1,0.1]]| units.m)))
        
    def test3(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        points = grid.points()
        self.assertEqual(points.shape, (6,5,3, 3))
        self.assertAlmostRelativeEquals(points[0][0][0],  ([0.0,0.0, 0.0] | units.m) )
        self.assertAlmostRelativeEquals(points[1][0][0] ,  ([0.2,0.0, 0.0] | units.m) )
        self.assertAlmostRelativeEquals(points[1][1][1],  [0.2,0.25, 0.5] | units.m )
        self.assertAlmostRelativeEquals(points[0][-1][-1] ,  ([0.0, 1.0, 1.0] | units.m) )
        self.assertAlmostRelativeEquals(points[-1][0][-1] ,  ([1.0, 0.0, 1.0] | units.m) )
        self.assertAlmostRelativeEquals(points[-1][-1][0] ,  ([1.0, 1.0, 0.0] | units.m) )
        self.assertAlmostRelativeEquals(points[-1][-1][-1] ,  ([1.0,1.0, 1.0] | units.m) )
        
    
    def test4(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        points = grid.points().reshape([6*5*3,3])
        connectivity = grid.connectivity()
        
        self.assertEqual(connectivity.shape, (5,4,2, 8))
        first_cell = connectivity[0][0][0]
        self.assertAlmostRelativeEquals(points[first_cell[0]], [0,0,0 ] | units.m)
        self.assertAlmostRelativeEquals(points[first_cell[1]], [0.2,0,0] | units.m)
        self.assertAlmostRelativeEquals(points[first_cell[2]] ,  ([0,0.25,0] | units.m))
        self.assertAlmostRelativeEquals(points[first_cell[3]] ,  ([0.2,0.25,0] | units.m))
        self.assertAlmostRelativeEquals(points[first_cell[4]] ,  ([0.0,0.0,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[first_cell[5]] ,  ([0.2,0.0,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[first_cell[6]] ,  ([0.0,0.25,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[first_cell[7]] ,  ([0.2,0.25,0.5] | units.m))
        
        self.assertEqual(connectivity[0][0][0], [ 0,15,  3, 18,  1, 16, 4, 19])
        
        
    
    def test5(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        points = grid.points().reshape([6*5*3,3])
        connectivity = grid.connectivity()
        
        self.assertEqual(connectivity.shape, (5,4,2, 8))
        cell = connectivity[0][0][1]
        self.assertAlmostRelativeEquals(points[cell[0]] ,  ([0.0,0.0,0.5 ] | units.m))
        self.assertAlmostRelativeEquals(points[cell[1]] ,  ([0.2,0,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[2]] ,  ([0,0.25,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[3]] ,  ([0.2,0.25,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[4]] ,  ([0.0,0.0,1.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[5]] ,  ([0.2,0.0,1.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[6]] ,  ([0.0,0.25,1.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[7]] ,  ([0.2,0.25,1.0] | units.m))
        
        self.assertEqual(connectivity[0][0][0], [ 0,15,  3, 18,  1, 16, 4, 19])
        
    
    
    def test6(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        points = grid.points().reshape([6*5*3,3])
        connectivity = grid.connectivity()
        
        self.assertEqual(connectivity.shape, (5,4,2, 8))
        cell = connectivity[1][1][1]
        self.assertAlmostRelativeEquals(points[cell[0]], ([0.2, 0.25, 0.5]|units.m) + ([0.0,0.0,0.0 ] | units.m))
        self.assertAlmostRelativeEquals(points[cell[1]], ([0.2, 0.25, 0.5]|units.m) + ([0.2,0,0.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[2]], ([0.2, 0.25, 0.5]|units.m) + ([0,0.25,0.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[3]], ([0.2, 0.25, 0.5]|units.m) + ([0.2,0.25,0.0] | units.m))
        self.assertAlmostRelativeEquals(points[cell[4]], ([0.2, 0.25, 0.5]|units.m) + ([0.0,0.0,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[5]], ([0.2, 0.25, 0.5]|units.m) + ([0.2,0.0,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[6]], ([0.2, 0.25, 0.5]|units.m) + ([0.0,0.25,0.5] | units.m))
        self.assertAlmostRelativeEquals(points[cell[7]], ([0.2, 0.25, 0.5]|units.m) + ([0.2,0.25,0.5] | units.m))
        
        self.assertEqual(connectivity[0][0][0], [ 0,15,  3, 18,  1, 16, 4, 19])
    
    def test7(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        
        self.assertAlmostRelativeEquals(grid[1][2][3].position, [3,5,7] |units.m)
        
        grid[1][2][3].position = [7,5,3] |units.m
        self.assertAlmostRelativeEquals(grid[1][2][3].position, [7,5,3] |units.m)
        
        
        grid[1][2][3].position += [1,2,3] |units.m
        self.assertAlmostRelativeEquals(grid[1][2][3].position, [8,7,6] |units.m)

    def test8(self):
        grid = datamodel.new_regular_grid((5,4), [1.0, 1.0] | units.m)
        self.assertAlmostRelativeEquals(grid.get_minimum_position(),  ([0.0, 0.0] | units.m) )
        self.assertAlmostRelativeEquals(grid.get_maximum_position(),  [1.0, 1.0] | units.m)
        self.assertAlmostRelativeEquals(grid.get_volume(),  1.0 | units.m ** 2)
        self.assertTrue(grid.contains([0.5,0.5] | units.m))

    def test9(self):
        grid = datamodel.new_regular_grid((5,4,2), [1.0, 1.0, 1.0] | units.m)
        self.assertEqual((0,0,0),grid.get_minimum_index())
        self.assertEqual((4,3,1),grid.get_maximum_index())

    def test10(self):
        grid1 = datamodel.new_regular_grid((5,4), [1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((5,4), [.1, .1] | units.m)
        grid3 = datamodel.new_regular_grid((5,4), [.1, .1] | units.m,offset=[0.5,0.6] | units.m)
        self.assertTrue(grid1.overlaps(grid2))
        self.assertTrue(grid1.overlaps(grid3))
        self.assertFalse(grid2.overlaps(grid3))
        self.assertTrue(grid2.overlaps(grid1))
        self.assertTrue(grid3.overlaps(grid1))
        self.assertFalse(grid3.overlaps(grid2))

    def test11(self):
        grid1 = datamodel.new_regular_grid((4,4), [1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((4,4), [1.0, 1.0] | units.m,offset=[-0.5,-0.5] | units.m)
        self.assertTrue(grid1.overlaps(grid2))
        overlap=grid1.get_overlap_with(grid2)
        self.assertEqual(overlap.position,grid1[0:3,0:3].position)

    def test12(self):
        grid1 = datamodel.new_regular_grid((4,4), [1.0, 1.0] | units.m)
        grid2 = datamodel.new_regular_grid((4,4), [1.0, 1.0] | units.m,offset=[-0.5,-0.5] | units.m)
        self.assertTrue(grid1.overlaps(grid2))
        overlap=grid1.get_overlap_with(grid2,eps=grid2.cellsize()[0]*1.e-6)
        self.assertEqual(overlap.position,grid1[0:2,0:2].position)


class TestGridSampling(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        self.assertEqual(sample.index , [1,1,1])
        sample = grid.samplePoint([2.5,2.5,2.5]| units.m,method="interpolation")
        self.assertEqual(sample.index , [1,1,1])
        sample = grid.samplePoint([3.5,3.5,3.5]| units.m,method="interpolation")
        self.assertEqual(sample.index , [1,1,1])
        
        for x in range(0,200):
            sample = grid.samplePoint([0.0 + (x/100.0),4.0+(x/100.0),6.0+(x/100.0)]| units.m,method="interpolation")
            self.assertEqual(sample.index , [0,2,3])
            
        for x in range(200,400):
            sample = grid.samplePoint([0.0 + (x/100.0),4.0+(x/100.0),6.0+(x/100.0)]| units.m,method="interpolation")
            self.assertEqual(sample.index , [1,3,4])

    def test2(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        self.assertEqual(sample.index_for_000_cell , [1,1,1])
        sample = grid.samplePoint([2.5,2.5,2.5]| units.m,method="interpolation")
        self.assertEqual(sample.index_for_000_cell , [0,0,0])
        sample = grid.samplePoint([3.5,3.5,3.5]| units.m,method="interpolation")
        self.assertEqual(sample.index_for_000_cell , [1,1,1])
        sample = grid.samplePoint([4.5,4.5,4.5]| units.m,method="interpolation")
        self.assertEqual(sample.index_for_000_cell , [1,1,1])
        self.assertEqual(sample.index , [2,2,2])
        
        for x in range(0,100):
            
            sample = grid.samplePoint([0.0 + (x/100.0),4.0+(x/100.0),6.0+(x/100.0)]| units.m,method="interpolation")
            self.assertEqual(sample.index_for_000_cell , [-1,1,2])
        for x in range(100,300):
            
            sample = grid.samplePoint([0.0 + (x/100.0),4.0+(x/100.0),6.0+(x/100.0)]| units.m,method="interpolation")
            self.assertEqual(sample.index_for_000_cell , [0,2,3])
            
        for x in range(300,400):
            sample = grid.samplePoint([0.0 + (x/100.0),4.0+(x/100.0),6.0+(x/100.0)]| units.m,method="interpolation")
            self.assertEqual(sample.index_for_000_cell , [1,3,4])
            
    
    def test3(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        self.assertEqual(sample.index_for_000_cell , [1,1,1])
        self.assertEqual(sample.surrounding_cell_indices , [
            [1,1,1],
            [2,1,1],
            [1,2,1],
            [1,1,2],
            [2,1,2],
            [1,2,2],
            [2,2,1],
            [2,2,2],
        ])
    
    def test4(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        self.assertEqual(sample.surrounding_cells[0].position , [3.0,3.0,3.0] | units.m )    
        self.assertEqual(sample.surrounding_cells[1].position , [5.0,3.0,3.0] | units.m )   
        self.assertEqual(sample.surrounding_cells[-1].position , [5.0,5.0,5.0] | units.m )        

    
    def test5(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        masses = sample.get_values_of_attribute("mass")
        self.assertEqual(masses[0] , 3.0 | units.kg ) 
        self.assertEqual(masses[1] , 5.0 | units.kg ) 
        self.assertEqual(masses[2] , 3.0 | units.kg )  
        self.assertEqual(masses[3] , 3.0 | units.kg )  
        self.assertEqual(masses[4] , 5.0 | units.kg )  
        self.assertEqual(masses[5] , 3.0 | units.kg )  
        self.assertEqual(masses[6] , 5.0 | units.kg )  
        self.assertEqual(masses[7] , 5.0 | units.kg ) 
        factors = sample.weighing_factors
        self.assertEqual(factors[0] , 1.0 | units.none ) 
        self.assertEqual(factors[1] , 0.0 | units.none ) 
        self.assertEqual(factors[2] , 0.0 | units.none )  
        self.assertEqual(factors[3] , 0.0 | units.none )  
        self.assertEqual(factors[4] , 0.0 | units.none )  
        self.assertEqual(factors[5] , 0.0 | units.none )  
        self.assertEqual(factors[6] , 0.0 | units.none )  
        self.assertEqual(factors[7] , 0.0 | units.none ) 
        
        self.assertAlmostRelativeEquals(sample.mass , 3.0 | units.kg ) 
            
    def test6(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        for xpos in numpy.arange(3.0,5.0,0.1):
            sample = grid.samplePoint([xpos,3.0,3.0]| units.m,method="interpolation")
            self.assertAlmostRelativeEquals(sample.mass , (3.0 | units.kg) + ((2.0 * (xpos - 3.0) / 2.0) | units.kg) ) 
          
            sample = grid.samplePoint([xpos,3.0,3.0]| units.m,method="interpolation")
            self.assertAlmostRelativeEquals(sample.mass , (3.0 | units.kg) + ((2.0 * (xpos - 3.0) / 2.0) | units.kg) ) 
          
            sample = grid.samplePoint([xpos,5.0,3.0]| units.m,method="interpolation")
            self.assertAlmostRelativeEquals(sample.mass , (3.0 | units.kg) + ((2.0 * (xpos - 3.0) / 2.0) | units.kg) ) 
          
            sample = grid.samplePoint([xpos,3.0,5.0]| units.m,method="interpolation")
            self.assertAlmostRelativeEquals(sample.mass , (3.0 | units.kg) + ((2.0 * (xpos - 3.0) / 2.0) | units.kg) ) 
        

            sample = grid.samplePoint([4.0,4.0,4.0]| units.m,method="interpolation")
            self.assertAlmostRelativeEquals(sample.mass , (4.0 | units.kg)) 

    def test7(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m,method="interpolation")
        self.assertTrue(sample.isvalid)
        sample = grid.samplePoint([11.0,3.0,3.0]| units.m,method="interpolation")
        self.assertFalse(sample.isvalid)
        sample = grid.samplePoint([3.0,-1.0,3.0]| units.m,method="interpolation")
        self.assertFalse(sample.isvalid)
        
    
    def test8(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m, method="nearest")    
        self.assertEqual(sample.position, [3.0,3.0,3.0]| units.m)    
        self.assertEqual(sample.mass, 3.0 | units.kg)
        sample = grid.samplePoint([3.5,3.0,3.0]| units.m, method="nearest")    
        self.assertEqual(sample.position, [3.0,3.0,3.0]| units.m)  
        self.assertEqual(sample.mass, 3.0 | units.kg)
        
    def test9(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        sample = grid.samplePoint([3.0,3.0,3.0]| units.m, method="linear")    
        self.assertEqual(sample.position, [3.0,3.0,3.0]| units.m)    
        self.assertEqual(sample.mass, 3.0 | units.kg)
        sample = grid.samplePoint([3.5,3.0,3.0]| units.m, method="linear")    
        self.assertEqual(sample.position, [3.5,3.0,3.0]| units.m)  
        self.assertEqual(sample.mass, 3.5 | units.kg)
        
    
class TestGridSamplingMultiplePoints(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        samples = grid.samplePoints([[3.0,3.0,3.0], [4.0,3.0,3.0]]| units.m, method="linear")
        self.assertEqual(len(samples), 2)
        self.assertEqual(samples.position[0] , [3.0,3.0,3.0]| units.m)
        self.assertEqual(samples.position[0] , samples[0].position)
        self.assertEqual(samples.position[1] , samples[1].position)
        self.assertEqual(samples.mass , [3.0, 4.0] | units.kg)
    
    def test2(self):
        grid = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid.mass = grid.x.value_in(units.m) | units.kg
        samples = grid.samplePoints([[3.5,3.0,3.0], [4.5,3.0,3.0]]| units.m, method="linear")
        self.assertEqual(len(samples), 2)
        self.assertEqual(samples.mass , [3.5, 4.5] | units.kg)
        
    def test3(self):
        grid1 = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid1.mass = grid1.x.value_in(units.m) | units.kg
        grid2 = datamodel.new_regular_grid((5,5,5), [10.0, 10.0, 10.0] | units.m)
        grid2.position += (10.0,0,0) | units.m
        grid2.mass = grid2.x.value_in(units.m) | units.kg
        samples = SamplePointsOnMultipleGrids((grid1, grid2), [[3.0,3.0,3.0], [4.0,3.0,3.0], [13,3,3]]| units.m)
        self.assertEqual(len(samples), 3)
        self.assertEqual(samples.mass , [3.0, 4.0, 13.0] | units.kg)



import os.path
import numpy
from amuse.test.amusetest import TestWithMPI

from amuse.community.krome.interface import KromeInterface,Krome,solar_abundances
from amuse.units import units
from amuse.datamodel import Particles

from amuse.io import read_set_from_file

#default_options={}
default_options = dict(redirection="none")
#default_options=dict(debugger="gdb")

class TestKromeInterface(TestWithMPI):

    def test1(self):
        print("Test 1: initialization")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())
        self.assertEqual(0, instance.cleanup_code())
        instance.stop()

    def test2(self):
        print("Test 1: add particle, get state")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e5
        t=500.
        ion=1.e-11
        id,err=instance.new_particle(dens,t,ion)

        self.assertEqual(err,0)
      
        self.assertEqual(instance.commit_particles(),0)

        dens_,t_,ion_,err=instance.get_state(id)

        self.assertEqual(err,0)

        self.assertEqual(dens_,dens)
        self.assertEqual(t_,t)
        self.assertEqual(ion_,ion)

        self.assertEqual(0, instance.cleanup_code())

        instance.stop()
        
    def test3(self):
        print("Test 1: add 2 particles, get state")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=[1.e5,2.e5]
        t=[500.,550]
        ion=[1.e-11,2.e-11]
        id,err=instance.new_particle(dens,t,ion)

        self.assertEqual(err,0)
      
        self.assertEqual(instance.commit_particles(),0)

        for i in range(2):
          dens_,t_,ion_,err=instance.get_state(id[i])

          self.assertEqual(err,0)

          self.assertEqual(dens_,dens[i])
          self.assertEqual(t_,t[i])
          self.assertEqual(ion_,ion[i])

        self.assertEqual(0, instance.cleanup_code())

        instance.stop()

    def test4(self):
        print("Test 1: add 100 particles, get state")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e5*numpy.random.random(100)
        t=500.*numpy.random.random(100)
        ion=1.e-11*numpy.random.random(100)
        id,err=instance.new_particle(dens,t,ion)

        self.assertEqual(err,0)
      
        self.assertEqual(instance.commit_particles(),0)

        for i in range(100):
          dens_,t_,ion_,err=instance.get_state(id[i])

          self.assertEqual(err,0)

          self.assertEqual(dens_,dens[i])
          self.assertEqual(t_,t[i])
          self.assertEqual(ion_,ion[i])

        self.assertEqual(0, instance.cleanup_code())

        instance.stop()

    def test5(self):
        print("Test 5: can we get species?")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        
        first,last,err=instance.get_firstlast_abundance()
        self.assertEqual(err,0)
        self.assertTrue(last-first > 0)
        
        for i in range(first,last+1):
          name,err=instance.get_name_of_species(i)
          print(name)
          self.assertEqual(err,0)
          index,err=instance.get_index_of_species(name)
          self.assertEqual(i,index)

        instance.stop()

    def test6(self):
        print("Test 6: add 100 particles, remove particles")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e5*numpy.random.random(100)
        t=500.*numpy.random.random(100)
        ion=1.e-11*numpy.random.random(100)
        ids,err=instance.new_particle(dens,t,ion)

        self.assertEqual(err,0)
      
        self.assertEqual(instance.commit_particles(),0)

        for i in ids[:10]:
          instance.delete_particle(i)

        instance.recommit_particles()

        for i in range(10,100):
          dens_,t_,ion_,err=instance.get_state(ids[i])

          self.assertEqual(err,0)

          self.assertEqual(dens_,dens[i])
          self.assertEqual(t_,t[i])
          self.assertEqual(ion_,ion[i])

        dens_,t_,ion_,err=instance.get_state(ids[0])

        self.assertEqual(err,-1)

        self.assertEqual(0, instance.cleanup_code())

        instance.stop()

    def test7(self):
        print("Test 1: add particle, set abundances")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e5
        t=500.
        ion=1.e-11
        id,err=instance.new_particle(dens,t,ion)
 
        instance.commit_particles()

        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertTrue((x>=0.) & (x<=1.))
          self.assertEqual(err,0)
        x,err=instance.get_abundance(id,last+1)
        self.assertEqual(err,-1)
        
        for s in ["H","HE","C","SI","O"]:
          x=solar_abundances[s]
          aid,err=instance.get_index_of_species(s)          
          instance.set_abundance(id,aid,x)

        for s in ["H","HE","C","SI","O"]:
          x=solar_abundances[s]
          aid,err=instance.get_index_of_species(s)          
          xx,err=instance.get_abundance(id,aid)
          self.assertEqual(x,xx)
          self.assertEqual(err,0)
          
    def test8(self):
        print("evolve test")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e2
        t=100.
        ion=2.e-17
        id,err=instance.new_particle(dens,t,ion)
         
        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          err=instance.set_abundance(id,i,1.e-40)         
         
        for s in ["H","HE","C","SI","O"]:
          x=solar_abundances[s]
          aid,err=instance.get_index_of_species(s)          
          instance.set_abundance(id,aid,x)

        instance.commit_particles()

        yr=365*24*3600.
        err=instance.evolve_model(10000.*yr)
        self.assertEqual(err,0)
        time,err=instance.get_time()
        self.assertEqual(err,0)
        self.assertEqual(time,10000.*yr)

        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertEqual(err,0)
          name,err=instance.get_name_of_species(i)
          print(i,name,x)
        
    def test9(self):
        print("evolve test 2")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e2
        t=100.
        ion=2.e-17
        id,err=instance.new_particle(dens,t,ion)
         
        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          err=instance.set_abundance(id,i,1.e-40)         
         
        for s in ["H","HE","C","SI","O"]:
          x=solar_abundances[s]
          aid,err=instance.get_index_of_species(s)          
          instance.set_abundance(id,aid,x)

        aid,err=instance.get_index_of_species("C")          
        instance.set_abundance(id,aid,1.e-40)
        
        aid,err=instance.get_index_of_species("C+")          
        instance.set_abundance(id,aid,solar_abundances["C"])

        aid,err=instance.get_index_of_species("H2")          
        instance.set_abundance(id,aid,1.e-6)
        aid,err=instance.get_index_of_species("H+")          
        instance.set_abundance(id,aid,1.e-4)

        instance.commit_particles()

        yr=365*24*3600.
        err=instance.evolve_model(10000.*yr)
        self.assertEqual(err,0)
        time,err=instance.get_time()
        self.assertEqual(err,0)
        self.assertEqual(time,10000.*yr)

        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertEqual(err,0)
          name,err=instance.get_name_of_species(i)
          print(i,name,x)

    def test10(self):
        print("check initialization of abundances")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e2
        t=100.
        ion=2.e-17
        id,err=instance.new_particle(dens,t,ion)
        
        abundances={"E":0.000369180975425,
                     "H+":0.0001,"HE":0.0775, 
                     "C+":0.000269180975425, "SI": 3.2362683404e-05,
                     "O": 0.000489828841345}
         
        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertEqual(err,0)
          name,err=instance.get_name_of_species(i)
          if name in abundances:
            self.assertAlmostEqual(x,abundances[name],12)


    def test11(self):
        print("evolve test, comparison")

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e2
        t=100.
        ion=2.e-17
        id,err=instance.new_particle(dens,t,ion)
        instance.evolve_model(1.e10)
        
        result1={}
        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertEqual(err,0)
          name,err=instance.get_name_of_species(i)
          result1[name]=x

        instance = self.new_instance_of_an_optional_code(KromeInterface, **default_options)
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        dens=1.e2
        t=100.
        ion=2.e-17
        id,err=instance.new_particle(dens,t,ion)
        instance.evolve_model(1.e9)
        instance.evolve_model(5.e9)
        instance.evolve_model(1.e10)
        
        result2={}
        first,last,err=instance.get_firstlast_abundance()
        for i in range(first,last+1):
          x,err=instance.get_abundance(id,i)
          self.assertEqual(err,0)
          name,err=instance.get_name_of_species(i)
          result2[name]=x

        for x in result1:
          self.assertAlmostEqual(result1[x],result2[x])


class TestKrome(TestWithMPI):
    def makeparts(self,N):
        parts=Particles(N)
        numpy.random.seed(1234567)
        parts.number_density=(numpy.random.random(N)*1.e5+1.e5)| units.cm**-3
        parts.temperature=(numpy.random.random(N)*500+100)| units.K
        parts.ionrate=(numpy.random.random(N)*1.e-11+1.e-17)| units.s**-1
        return parts

    def test0(self):
        print("test1: basic startup and flow")
        instance=self.new_instance_of_an_optional_code(Krome)
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.initialize_code()
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        instance.commit_parameters()
        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')
        instance.commit_particles()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')

        instance.cleanup_code()
        instance.stop()

    def test1(self):
        print("test1: adding particles")

        instance=self.new_instance_of_an_optional_code(Krome)

        parts=self.makeparts(5)
                
        self.assertEqual(len(instance.particles),0)
        instance.particles.add_particles(parts)
        self.assertEqual(len(instance.particles),len(parts))

        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')

        part2=instance.particles.copy()

        self.assertAlmostRelativeEquals(parts.number_density,part2.number_density,12)
        self.assertAlmostRelativeEquals(parts.temperature,part2.temperature,12)
        self.assertAlmostRelativeEquals(parts.ionrate,part2.ionrate,12)

        for p in part2:
          i=instance.species["E"]
          self.assertAlmostEqual(p.abundances[i],0.000369180975425)
          i=instance.species["H+"]
          self.assertAlmostEqual(p.abundances[i],0.0001)
          i=instance.species["HE"]
          self.assertAlmostEqual(p.abundances[i],0.0775)
          i=instance.species["C+"]
          self.assertAlmostEqual(p.abundances[i],0.000269180975425)
          i=instance.species["SI"]
          self.assertAlmostEqual(p.abundances[i],3.2362683404e-05)
          i=instance.species["O"]
          self.assertAlmostEqual(p.abundances[i],0.000489828841345)
          
        instance.cleanup_code()
        instance.stop()


    def test2(self):
        print("test2: adding particles w abund.")

        instance=self.new_instance_of_an_optional_code(Krome)

        parts=self.makeparts(5)
        
        N=len(instance.species)

        parts.abundances=numpy.zeros((5,N))        

        for i in range(5):
          parts[i].abundances=(numpy.array(range(N))+1)/(N+1.)
 
        instance.particles.add_particles(parts)

        channel=parts.new_channel_to(instance.particles)
        channel.copy() 

        part2=instance.particles.copy()

        self.assertAlmostRelativeEquals(parts.number_density,part2.number_density,12)
        self.assertAlmostRelativeEquals(parts.temperature,part2.temperature,12)
        self.assertAlmostRelativeEquals(parts.ionrate,part2.ionrate,12)

        for i in range(5):  
          self.assertAlmostRelativeEquals(part2[i].abundances,parts[i].abundances ,12)

        instance.cleanup_code()
        instance.stop()

    def test3(self):
        print("test3: evolve test")

        instance=self.new_instance_of_an_optional_code(Krome,**default_options)

        parts=Particles(1)
        parts.number_density=1.e5 | units.cm**-3
        parts.temperature=50 | units.K
        parts.ionrate=2.e-17 | units.s**-1

        Ns=len(instance.species)

        parts.abundances=numpy.zeros((1,Ns))        

        instance.particles.add_particles(parts)
  
        instance.evolve_model( 1. | units.Myr )
 
        print(instance.particles.abundances)

        f=2*instance.particles[0].abundances[instance.species["H2"]]
        self.assertTrue(f> 0.95) # not much of a test..
        #~ for x,i in instance.species.items():
          #~ print x, instance.particles[0].abundances[i]
        
        instance.cleanup_code()
        instance.stop()

    def test4(self):
        print("test4: evolve test (10 part)")

        instance=self.new_instance_of_an_optional_code(Krome,**default_options)

        parts=Particles(10)
        parts.number_density=1.e5 | units.cm**-3
        parts.temperature=50 | units.K
        parts.ionrate=2.e-17 | units.s**-1

        Ns=len(instance.species)
        
        parts.abundances=numpy.zeros((10,Ns))        

        instance.particles.add_particles(parts)
  
        instance.evolve_model( 1. | units.Myr )

        f=2*instance.particles[0].abundances[instance.species["H2"]]
        self.assertTrue(f> 0.95) # not much of a test..
        #~ for x,i in instance.species.items():
          #~ print x, instance.particles[0].abundances[i]
        
        instance.cleanup_code()
        instance.stop()
